/**
 * Copyright (c) 2008, http://www.snakeyaml.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.yaml.snakeyaml.immutable;

public class SuperColor {
    private final Color color;

    public SuperColor(Color name) {
        this.color = name;
    }

    public Color getName() {
        return color;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof SuperColor) {
            SuperColor color = (SuperColor) obj;
            return color.equals(color.color);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        return color.hashCode();
    }

    @Override
    public String toString() {
        return "SuperColor color=" + color;
    }

    public Color getColor() {
        return color;
    }
}
